#ifndef VECTOR3_H
#define VECTOR3_H

#include <math.h>

/** A 3D double-precision Vector class and its operations. */
class Vector3
{
public:
    /** Default constructor, initializes to (0,0,0). */
    Vector3() {
        x = y = z = 0;
    }

    /** Constructs a Vector3 with the given x,y,z coordinates. */
    Vector3(double px, double py, double pz) : x(px), y(py), z(pz) {}
        /** Returns the length of the vector. */
    double Length() const {
        return sqrt(x*x + y*y + z*z);
    }

    /** Returns the squared length of the vector, which is faster than computing
    the length. */
    double SquaredLength() const {
        return (x*x + y*y + z*z);
    }

    /** Scales the vector to be of length 1.0. */
    void Normalize()
    {
        double n = 1.0 / Length();
        x *= n;
        y *= n;
        z *= n;
    }

    /** Scales each x,y,z value of the vector by a constant n, and returns the result. */
    Vector3 operator * (double n) const
    {
        return Vector3(x*n, y*n, z*n);
    }

    /** Adds this vector to the specified vector, and returns the result. */
    Vector3 operator + (const Vector3& v) const
    {
        return Vector3(x + v.x, y + v.y, z + v.z);
    }

    /** Subtracts the specified vector from this vector, and returns the result. */
    Vector3 operator - (const Vector3& v) const
    {
        return Vector3(x - v.x, y - v.y, z- v.z);
    }

    /** Data members x,y,z public for convenience. */
    double x, y, z;
};

#endif
